package node.stmtNode;

import env.*;
import code.*;

import node.exprNode.*;

public class ForStmtNode extends StmtNode {

	private String ident;
	private ExprNode initExpr;
	private ExprNode finalExpr;
	private StmtNode stmt;

	public ForStmtNode( 
		String ident, 
		ExprNode initExpr, 
		ExprNode finalExpr, 
		StmtNode stmt ) {
		this.ident = ident;
		this.initExpr = initExpr;
		this.finalExpr = finalExpr;
		this.stmt = stmt;
		}

	public String toString() {
		return "for " + ident + " = " + initExpr + " to " + finalExpr 
			+ " do%+%n" + stmt + "%-";
		}

	public void genDeclCode( Env env ) {
		env.add( ident );
		stmt.genDeclCode( env );
		}
		
	public void genCode() {
		Code.enter();
		Code.labelDefn( "for" );
		initExpr.evalCode( 0 );
		Code.instrn( "ldiq", "$t1", ident );
		Code.instrn( "stq", "$t0", "($t1)" );
		Code.labelDefn( "while" );
		Code.instrn( "ldiq", "$t0", ident );
		Code.instrn( "ldq", "$t0", "($t0)" );
		finalExpr.evalCode( 1 );
		Code.instrn( "cmple", "$t0", "$t1" );
		Code.instrn( "blbc", "$t0", "end" );
		Code.labelDefn( "do" );
		stmt.genCode();
		Code.labelDefn( "increment" );
		Code.instrn( "ldiq", "$t0", ident );
		Code.instrn( "ldq", "$t1", "($t0)" );
		Code.instrn( "addq", "$t1", "1" );
		Code.instrn( "stq", "$t1", "($t0)" );
		Code.instrn( "br", "while" );
		Code.labelDefn( "end" );
		Code.exit();
		}
	}
