import java.io.*;
import java_cup.runtime.*;
import node.*;
import text.*;
import grammar.*;

public class Main {

	public static void main( String[] argv ) {
		String dirName = null;

		try {
			for ( int i = 0; i < argv.length; i++ ) {
				if ( argv[ i ].equals( "-debug" ) ) {
					Print.DEBUG = true;
					}
				else if ( argv[ i ].equals( "-dir" ) ) {
					i++;
					if ( i >= argv.length )
						throw new Error( "Missing directory name" );
					dirName = argv[ i ];
					}
				else {
					throw new Error( 
						"Usage: java Main [-debug] -dir directory" );
					}
				}

			if ( dirName == null )
				throw new Error( "Directory not specified" );

			Print.setError( new File( dirName, "program.err" ) );
			Print.setReprint( new File( dirName, "program.print" ) );

			parser programParser = new parser( 
				new File( dirName, "program.in" ) );
			ProgramNode program;
			if ( Print.DEBUG )
				program = ( ProgramNode ) programParser.debug_parse().value;
			else
				program = ( ProgramNode ) programParser.parse().value;

			Print.error().println( "Reprinting ... " );
			Print.reprint().print( program );
			}
		catch ( Throwable exception ) {
			exception.printStackTrace();
			Print.error().println( "Exception in Main " + exception );
			Print.error().printStackTrace( exception );
			System.exit( -1 );
			}
		}
	}

