package grammar;

import text.*;
import java_cup.runtime.*;

import java.io.*;
import java.util.*;

%%

%public
%type		Symbol
%char

%{

	private int lineNumber = 1;
	public int lineNumber() { return lineNumber; }
	
	public Symbol token( int tokenType, String text ) {
		Print.error().debugln( 
			"Obtain token " + sym.terminal_name( tokenType ) + " \"" + text + "\"" );
		return new Symbol( tokenType, yychar, yychar + yytext().length(), text );
		}
	
	public Symbol token( int tokenType ) {
		return token( tokenType, yytext() );
		}
	
%}

%init{
	yybegin( NORMALSTATE );
%init}

intConst	=	[0-9]+
name		=	[a-z][A-Za-z0-9]*
variable	=	[A-Z][A-Za-z0-9]*
space		=	[\ \t]
newline		=	\r|\n|\r\n
%state	NORMALSTATE, COMMENT1STATE, COMMENT2STATE, LEXERRORSTATE

%%
<NORMALSTATE> {

	{newline}	{ lineNumber++; }
	{space}		{ }
	
	"."			{ return token( sym.DOT ); }
	":-"		{ return token( sym.EXPANDSTO ); }
	","			{ return token( sym.COMMA ); }
	"|"			{ return token( sym.BAR ); }
	
	"("			{ return token( sym.LEFT ); }
	")"			{ return token( sym.RIGHT ); }
	
	"["			{ return token( sym.LEFTSQ ); }
	"]"			{ return token( sym.RIGHTSQ ); }
	
	{intConst}	{ return token( sym.INTVALUE ); }
	{name}		{ return token( sym.NAME ); }
	{variable}	{ return token( sym.VARIABLE ); }
	
	"%"			{ yybegin( COMMENT1STATE ); }
	"/*"		{ yybegin( COMMENT2STATE ); }
	
	.			{ yybegin( LEXERRORSTATE ); return token( sym.LEXERROR ); }
	}

<LEXERRORSTATE> {
	{newline}	{ lineNumber++; yybegin( NORMALSTATE ); return token( sym.LEXERROR ); }
	.			{ }
	}

<COMMENT1STATE> {
	{newline}	{ lineNumber++; yybegin( NORMALSTATE ); }
	.			{ }
	}
	
<COMMENT2STATE> {
	"*/"		{ yybegin( NORMALSTATE ); }
	{newline}	{ lineNumber++; }
	.			{ }
	}

<<EOF>>			{ return token( sym.EOF ); }

